// CalendarDate.java
//
// ICS 22 / CSE 22 Fall 2012
// Code Example
//
// This is a simplistic implementation of a class whose objects
// each keep track of a calendar date.  Its key feature is that
// it implements the Comparable interface -- described in the
// comments of Comparable.java elsewhere in this example --
// meaning that we intend to allow our dates to be comparable
// to one another for the purposes of ordering.  In other words,
// we intend to be able to ask one CalendarDate: "How do you compare
// to this other CalendarDate?" and get back one of the following
// three answers:
//
//   - I'm smaller than the other one (earlier on the calendar)
//   - I'm larger than the other one (later on the calendar)
//   - I'm equivalent to the other one (same day on the calendar)
//
// Provided that we can ask these questions of any CalendarDate,
// and provided that the answers we get back are reasonable --
// they're always the same for any pair of dates, and the "smaller"
// and "larger" answers are transitive on three or more dates --
// it is possible to do some interesting things with lists of dates,
// such as sort them or find their minimum or maximum.
//
// The reason for the interface is that we'd like every kind of
// object that is comparable in this way to follow the same set
// of rules for performing the comparisons: using a method with
// the same name and signature that also has the same "meaning."
// If every comparable object plays by the same set of rules,
// we can use polymorphism to write algorithms such as "find
// the minimum" once, rather than separately for each kind of
// object.

public class CalendarDate implements Comparable
{
	private int year;
	private int month;
	private int day;

	
	public CalendarDate(int year, int month, int day)
	{
		this.year = year;
		this.month = month;
		this.day = day;
	}
	

	public int getYear()
	{
		return year;
	}
	
	
	public int getMonth()
	{
		return month;
	}
	
	
	public int getDay()
	{
		return day;
	}
	
	
	public String toString()
	{
		return String.format(
			"%04d-%02d-%02d",
			year, month, day);
	}


	// compareTo() compares this date to another object,
	// returning one of three possible answers:
	//
	//   * A negative number, if this date is "smaller"
	//     (smaller means earlier on the calendar)
	//   * A positive number, if this date is "larger"
	//     (larger means later on the calendar)
	//   * Zero, if the dates are the same
	public int compareTo(Object x)
	{
		// Because we were passed an Object reference, we need to first
		// establish that it really points to a CalendarDate.  We're fine
		// with the fact that this will cause a ClassCastException if x
		// is really pointing to another kind of object, because that's
		// what our interface says we should do when given an incompatible
		// parameter.		
		CalendarDate d = (CalendarDate) x;
		
		// There are a variety of better ways to solve this problem,
		// but this suffices for our purposes.
		if (year < d.year)
			return -1;
		else if (year > d.year)
			return 1;
		else if (month < d.month)
			return -1;
		else if (month > d.month)
			return 1;
		else if (day < d.day)
			return -1;
		else if (day > d.day)
			return 1;
		else
			return 0;
	}
}
